# Third-party Code Usages

This project directly uses some code from open source projects, the relevant license information is provided below. Relevant comments have been also attached to  code.

If we fail to update in time, please help to remind us. Thank you for your cooperation.

- **[better-exceptions](https://github.com/Qix-/better-exceptions)**
    - Referenced at: [_render.py](./src/melobot/_render.py)
    - License type: MIT
    - License web archive: https://github.com/Qix-/better-exceptions/blob/master/LICENSE.txt
    - License full text: 
        ```text
        The MIT License (MIT)

        Copyright (c) 2016 Josh Junon

        Permission is hereby granted, free of charge, to any person obtaining a copy
        of this software and associated documentation files (the "Software"), to deal
        in the Software without restriction, including without limitation the rights
        to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
        copies of the Software, and to permit persons to whom the Software is
        furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be included in
        all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
        IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
        AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
        LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
        OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
        THE SOFTWARE.
        ```

- **[Python](https://github.com/python/cpython)**
    - Referenced at: [_render.py](./src/melobot/_render.py)
    - License type: Python Software Foundation custom types
    - License web archive: https://github.com/python/cpython/blob/main/LICENSE
    - License full text: 
        ```text
        A. HISTORY OF THE SOFTWARE
        ==========================

        Python was created in the early 1990s by Guido van Rossum at Stichting
        Mathematisch Centrum (CWI, see https://www.cwi.nl) in the Netherlands
        as a successor of a language called ABC.  Guido remains Python's
        principal author, although it includes many contributions from others.

        In 1995, Guido continued his work on Python at the Corporation for
        National Research Initiatives (CNRI, see https://www.cnri.reston.va.us)
        in Reston, Virginia where he released several versions of the
        software.

        In May 2000, Guido and the Python core development team moved to
        BeOpen.com to form the BeOpen PythonLabs team.  In October of the same
        year, the PythonLabs team moved to Digital Creations, which became
        Zope Corporation.  In 2001, the Python Software Foundation (PSF, see
        https://www.python.org/psf/) was formed, a non-profit organization
        created specifically to own Python-related Intellectual Property.
        Zope Corporation was a sponsoring member of the PSF.

        All Python releases are Open Source (see https://opensource.org for
        the Open Source Definition).  Historically, most, but not all, Python
        releases have also been GPL-compatible; the table below summarizes
        the various releases.

            Release         Derived     Year        Owner       GPL-
                            from                                compatible? (1)

            0.9.0 thru 1.2              1991-1995   CWI         yes
            1.3 thru 1.5.2  1.2         1995-1999   CNRI        yes
            1.6             1.5.2       2000        CNRI        no
            2.0             1.6         2000        BeOpen.com  no
            1.6.1           1.6         2001        CNRI        yes (2)
            2.1             2.0+1.6.1   2001        PSF         no
            2.0.1           2.0+1.6.1   2001        PSF         yes
            2.1.1           2.1+2.0.1   2001        PSF         yes
            2.1.2           2.1.1       2002        PSF         yes
            2.1.3           2.1.2       2002        PSF         yes
            2.2 and above   2.1.1       2001-now    PSF         yes

        Footnotes:

        (1) GPL-compatible doesn't mean that we're distributing Python under
            the GPL.  All Python licenses, unlike the GPL, let you distribute
            a modified version without making your changes open source.  The
            GPL-compatible licenses make it possible to combine Python with
            other software that is released under the GPL; the others don't.

        (2) According to Richard Stallman, 1.6.1 is not GPL-compatible,
            because its license has a choice of law clause.  According to
            CNRI, however, Stallman's lawyer has told CNRI's lawyer that 1.6.1
            is "not incompatible" with the GPL.

        Thanks to the many outside volunteers who have worked under Guido's
        direction to make these releases possible.


        B. TERMS AND CONDITIONS FOR ACCESSING OR OTHERWISE USING PYTHON
        ===============================================================

        Python software and documentation are licensed under the
        Python Software Foundation License Version 2.

        Starting with Python 3.8.6, examples, recipes, and other code in
        the documentation are dual licensed under the PSF License Version 2
        and the Zero-Clause BSD license.

        Some software incorporated into Python is under different licenses.
        The licenses are listed with code falling under that license.


        PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2
        --------------------------------------------

        1. This LICENSE AGREEMENT is between the Python Software Foundation
        ("PSF"), and the Individual or Organization ("Licensee") accessing and
        otherwise using this software ("Python") in source or binary form and
        its associated documentation.

        2. Subject to the terms and conditions of this License Agreement, PSF hereby
        grants Licensee a nonexclusive, royalty-free, world-wide license to reproduce,
        analyze, test, perform and/or display publicly, prepare derivative works,
        distribute, and otherwise use Python alone or in any derivative version,
        provided, however, that PSF's License Agreement and PSF's notice of copyright,
        i.e., "Copyright (c) 2001 Python Software Foundation; All Rights Reserved"
        are retained in Python alone or in any derivative version prepared by Licensee.

        3. In the event Licensee prepares a derivative work that is based on
        or incorporates Python or any part thereof, and wants to make
        the derivative work available to others as provided herein, then
        Licensee hereby agrees to include in any such work a brief summary of
        the changes made to Python.

        4. PSF is making Python available to Licensee on an "AS IS"
        basis.  PSF MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
        IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, PSF MAKES NO AND
        DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
        FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF PYTHON WILL NOT
        INFRINGE ANY THIRD PARTY RIGHTS.

        5. PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON
        FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
        A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON,
        OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

        6. This License Agreement will automatically terminate upon a material
        breach of its terms and conditions.

        7. Nothing in this License Agreement shall be deemed to create any
        relationship of agency, partnership, or joint venture between PSF and
        Licensee.  This License Agreement does not grant permission to use PSF
        trademarks or trade name in a trademark sense to endorse or promote
        products or services of Licensee, or any third party.

        8. By copying, installing or otherwise using Python, Licensee
        agrees to be bound by the terms and conditions of this License
        Agreement.


        BEOPEN.COM LICENSE AGREEMENT FOR PYTHON 2.0
        -------------------------------------------

        BEOPEN PYTHON OPEN SOURCE LICENSE AGREEMENT VERSION 1

        1. This LICENSE AGREEMENT is between BeOpen.com ("BeOpen"), having an
        office at 160 Saratoga Avenue, Santa Clara, CA 95051, and the
        Individual or Organization ("Licensee") accessing and otherwise using
        this software in source or binary form and its associated
        documentation ("the Software").

        2. Subject to the terms and conditions of this BeOpen Python License
        Agreement, BeOpen hereby grants Licensee a non-exclusive,
        royalty-free, world-wide license to reproduce, analyze, test, perform
        and/or display publicly, prepare derivative works, distribute, and
        otherwise use the Software alone or in any derivative version,
        provided, however, that the BeOpen Python License is retained in the
        Software, alone or in any derivative version prepared by Licensee.

        3. BeOpen is making the Software available to Licensee on an "AS IS"
        basis.  BEOPEN MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
        IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BEOPEN MAKES NO AND
        DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
        FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT
        INFRINGE ANY THIRD PARTY RIGHTS.

        4. BEOPEN SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF THE
        SOFTWARE FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS
        AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE, OR ANY
        DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

        5. This License Agreement will automatically terminate upon a material
        breach of its terms and conditions.

        6. This License Agreement shall be governed by and interpreted in all
        respects by the law of the State of California, excluding conflict of
        law provisions.  Nothing in this License Agreement shall be deemed to
        create any relationship of agency, partnership, or joint venture
        between BeOpen and Licensee.  This License Agreement does not grant
        permission to use BeOpen trademarks or trade names in a trademark
        sense to endorse or promote products or services of Licensee, or any
        third party.  As an exception, the "BeOpen Python" logos available at
        http://www.pythonlabs.com/logos.html may be used according to the
        permissions granted on that web page.

        7. By copying, installing or otherwise using the software, Licensee
        agrees to be bound by the terms and conditions of this License
        Agreement.


        CNRI LICENSE AGREEMENT FOR PYTHON 1.6.1
        ---------------------------------------

        1. This LICENSE AGREEMENT is between the Corporation for National
        Research Initiatives, having an office at 1895 Preston White Drive,
        Reston, VA 20191 ("CNRI"), and the Individual or Organization
        ("Licensee") accessing and otherwise using Python 1.6.1 software in
        source or binary form and its associated documentation.

        2. Subject to the terms and conditions of this License Agreement, CNRI
        hereby grants Licensee a nonexclusive, royalty-free, world-wide
        license to reproduce, analyze, test, perform and/or display publicly,
        prepare derivative works, distribute, and otherwise use Python 1.6.1
        alone or in any derivative version, provided, however, that CNRI's
        License Agreement and CNRI's notice of copyright, i.e., "Copyright (c)
        1995-2001 Corporation for National Research Initiatives; All Rights
        Reserved" are retained in Python 1.6.1 alone or in any derivative
        version prepared by Licensee.  Alternately, in lieu of CNRI's License
        Agreement, Licensee may substitute the following text (omitting the
        quotes): "Python 1.6.1 is made available subject to the terms and
        conditions in CNRI's License Agreement.  This Agreement together with
        Python 1.6.1 may be located on the internet using the following
        unique, persistent identifier (known as a handle): 1895.22/1013.  This
        Agreement may also be obtained from a proxy server on the internet
        using the following URL: http://hdl.handle.net/1895.22/1013".

        3. In the event Licensee prepares a derivative work that is based on
        or incorporates Python 1.6.1 or any part thereof, and wants to make
        the derivative work available to others as provided herein, then
        Licensee hereby agrees to include in any such work a brief summary of
        the changes made to Python 1.6.1.

        4. CNRI is making Python 1.6.1 available to Licensee on an "AS IS"
        basis.  CNRI MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
        IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, CNRI MAKES NO AND
        DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
        FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF PYTHON 1.6.1 WILL NOT
        INFRINGE ANY THIRD PARTY RIGHTS.

        5. CNRI SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON
        1.6.1 FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
        A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON 1.6.1,
        OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

        6. This License Agreement will automatically terminate upon a material
        breach of its terms and conditions.

        7. This License Agreement shall be governed by the federal
        intellectual property law of the United States, including without
        limitation the federal copyright law, and, to the extent such
        U.S. federal law does not apply, by the law of the Commonwealth of
        Virginia, excluding Virginia's conflict of law provisions.
        Notwithstanding the foregoing, with regard to derivative works based
        on Python 1.6.1 that incorporate non-separable material that was
        previously distributed under the GNU General Public License (GPL), the
        law of the Commonwealth of Virginia shall govern this License
        Agreement only as to issues arising under or with respect to
        Paragraphs 4, 5, and 7 of this License Agreement.  Nothing in this
        License Agreement shall be deemed to create any relationship of
        agency, partnership, or joint venture between CNRI and Licensee.  This
        License Agreement does not grant permission to use CNRI trademarks or
        trade name in a trademark sense to endorse or promote products or
        services of Licensee, or any third party.

        8. By clicking on the "ACCEPT" button where indicated, or by copying,
        installing or otherwise using Python 1.6.1, Licensee agrees to be
        bound by the terms and conditions of this License Agreement.

                ACCEPT


        CWI LICENSE AGREEMENT FOR PYTHON 0.9.0 THROUGH 1.2
        --------------------------------------------------

        Copyright (c) 1991 - 1995, Stichting Mathematisch Centrum Amsterdam,
        The Netherlands.  All rights reserved.

        Permission to use, copy, modify, and distribute this software and its
        documentation for any purpose and without fee is hereby granted,
        provided that the above copyright notice appear in all copies and that
        both that copyright notice and this permission notice appear in
        supporting documentation, and that the name of Stichting Mathematisch
        Centrum or CWI not be used in advertising or publicity pertaining to
        distribution of the software without specific, written prior
        permission.

        STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
        THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
        FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
        FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
        WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
        ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
        OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

        ZERO-CLAUSE BSD LICENSE FOR CODE IN THE PYTHON DOCUMENTATION
        ----------------------------------------------------------------------

        Permission to use, copy, modify, and/or distribute this software for any
        purpose with or without fee is hereby granted.

        THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
        REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
        AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
        INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
        LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
        OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
        PERFORMANCE OF THIS SOFTWARE.
        ```

- **[rich](https://github.com/Textualize/rich)**
    - Referenced at: [_render.py](./src/melobot/_render.py)
    - License type: MIT
    - License web archive: https://github.com/Textualize/rich/blob/master/LICENSE
    - License full text: 
        ```text
        Copyright (c) 2020 Will McGugan

        Permission is hereby granted, free of charge, to any person obtaining a copy
        of this software and associated documentation files (the "Software"), to deal
        in the Software without restriction, including without limitation the rights
        to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
        copies of the Software, and to permit persons to whom the Software is
        furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be included in all
        copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
        IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
        AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
        LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
        OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
        SOFTWARE.
        ```

- **[websockets](https://github.com/python-websockets/websockets)**
    - Referenced at: [_lazy.py](./src/melobot/_lazy.py)
    - License type: BSD3
    - License web archive: https://github.com/python-websockets/websockets/blob/main/LICENSE
    - License full text:
        ```text
        Copyright (c) Aymeric Augustin and contributors

        Redistribution and use in source and binary forms, with or without
        modification, are permitted provided that the following conditions are met:

            * Redistributions of source code must retain the above copyright notice,
            this list of conditions and the following disclaimer.
            * Redistributions in binary form must reproduce the above copyright notice,
            this list of conditions and the following disclaimer in the documentation
            and/or other materials provided with the distribution.
            * Neither the name of the copyright holder nor the names of its contributors
            may be used to endorse or promote products derived from this software
            without specific prior written permission.

        THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
        ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
        WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
        DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
        FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
        DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
        SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
        CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
        OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
        OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
        ```
